# buildsystem.rb: Supporting library for icon artists, build system related
# stuff like adding icons to buildsystem or initializing a new one
#
# Copyright (C) 2008, 2009 Martin Sourada
# 
# This script is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This script is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this script; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

require "icon-artist/theme-settings"
require "fileutils"
require "rexml/document"

include FileUtils
include REXML

module IconArtist
  class BuildSystem
    private
    
    def initialize_build_sys
      puts "Creating new build sys in #{@dir}."
      FileUtils.mkdir_p(@dir) unless File.exists?(@dir)
      
      fin = IO.readlines("#{ARTIST_SHAREDIR}/buildsys/configure.src.in")
      fout = File.new("#{@dir}/configure.ac", "w+")
      fin.each do |line|
        line.gsub!('@theme-name@', @theme[:name])
        line.gsub!('@theme-version@', @theme[:version])
        line.gsub!('@theme-dir@', @theme[:dir])
        line.gsub!('@theme-bugzilla@', @theme[:bugzilla])
        fout.puts line
      end
      fout.close

      fin = IO.readlines("#{ARTIST_SHAREDIR}/buildsys/theme-create.rb.in")
      fout = File.new("#{@dir}/theme-create.rb", "w+")
      fin.each do |line|
        line.gsub!('@theme-name@', @theme[:name])
        line.gsub!('@theme-comment@', @theme[:comment])
        line.gsub!('@theme-inherits@', @theme[:inherits])
        fout.puts line
      end
      fout.close

      fin = IO.readlines("#{ARTIST_SHAREDIR}/buildsys/Makefile.theme.src.in")
      fout = File.new("#{@dir}/Makefile.am", "w+")
      fin.each do |line|
        fout.puts line
      end
      fout.close

      fin = IO.readlines("#{ARTIST_SHAREDIR}/buildsys/Makefile.src.in")
      if !File.exist?("#{@dir}/src")
        FileUtils.mkdir_p("#{@dir}/src")
      end
      fout = File.new("#{@dir}/src/Makefile.am", "w+")
      fin.each do |line|
        fout.puts line
      end
      fout.close

      if !File.exist? "#{@dir}/legacy-icon-mapping.dtd"
        FileUtils.cp("#{ARTIST_SHAREDIR}/buildsys/legacy-icon-mapping.dtd", "#{@dir}/legacy-icon-mapping.dtd")
      end

      if !File.exist? "#{@dir}/legacy-icon-mapping.xml"
        FileUtils.cp("#{ARTIST_SHAREDIR}/buildsys/legacy-icon-mapping.xml", "#{@dir}/legacy-icon-mapping.xml")
      end
      
      ['NEWS', 'README', 'ChangeLog', 'COPYING', 'AUTHORS'].each do |file|
        unless File.exist?("#{@dir}/#{file}")
          FileUtils.touch("#{@dir}/#{file}")
          puts "[Information] Empty #{file} file was created."
        end
      end
    end
    
    def check_for_theme
      if !File.exist?("#{@dir}/index.theme")
        if !File.exist?("#{@dir}/src/folder.svg")
          puts "Directory #{@dir} does not seem to hold an icon theme."
          initialize_build_sys
        end
      end
    end
    
    def create_symlinks(icon)
      # add symlinks
      symlinks = @icon.symlinks
      icon['sizes'].each do |size|
        if size == 'scalable'
          ext = '.svg'
        else
          ext = '.png'
        end
        symlinks[:default].each do |symlink|
          FileUtils.ln_sf("#{icon['name']}#{ext}", 
                          "#{@dir}/#{size}/#{icon['context']}/#{symlink}#{ext}")
        end
        symlinks[:optional].each do |symlink|
          FileUtils.ln_sf("#{icon['name']}#{ext}", 
                          "#{@dir}/#{size}/#{icon['context']}/#{symlink}#{ext}")
        end
      end
    end
    
    def add_icon_file_to_buildsys(lines, ext)
      icons_data = false
      icons = []
      is_in = false
      lines.each do |line|
        if line.include?("icons_DATA")
          icons_data = true
        elsif icons_data
          if line.include?("#{@icon.iconname}#{ext}")
            is_in = true
          end
          if !line.include?("\\")
            icon = line.rstrip.delete "\\"
            icon = icon.delete " "
            icons += [icon]
            icons_data = false
          else
            icon = line.rstrip.delete "\\"
            icon = icon.delete " "
            icons += [icon]
          end
        end
      end

      if !is_in
        icons = icons + ["\t#{@icon.iconname}#{ext}"]
      end

      icons.delete("")
      return icons.sort

    end
    
    def add_symlinks_to_buildsys(lines, ext)
      symlinks = @icon.symlinks[:optional]
      symlinks_new = []
      install_local = false; 

      lines.each do |line|
        if line.include?("install-data-local")
          install_local = true
        elsif install_local
          if line.rstrip == ""
            install_local = false
          else
            symlink = line
            symlinks.each do |link|
              if line.include?("#{link}#{ext}")
                symlinks = symlinks - [link]
              end
            end
            symlinks_new = symlinks_new + [symlink]
          end
        end
      end

      symlinks.each do |symlink|
        symlinks_new = symlinks_new + ["\t(cd $(DESTDIR)$(themedir)/$(size)/$(context) && $(LN_S) #{@icon.iconname}#{ext} #{symlink}#{ext})"]
      end
 
      return symlinks_new
    end
    
    def add_symlinks
      symlinks = @icon.symlinks[:optional]
      symlinks_old = []
      if symlinks != []
        symlinks_xml = "#{@dir}/legacy-icon-mapping.xml"
        if !File.exist?(symlinks_xml)
          symlinks_xml = "#{ARTIST_SHAREDIR}/buildsys/legacy-icon-mapping.xml.in"
          lines = IO.readlines("#{@dir}/Makefile.am")
          makefile = File.open("#{@dir}/Makefile.am", 'w')
          lines.each do |line|
            makefile.puts line
          end
        end
        xml = Document.new(File.new("#{symlinks_xml}", 'r'))
        context = xml.root.elements["//context[@dir='#{@icon.context}']"]
        if context != nil
          icon = context.elements["icon[@name='#{@icon.iconname}']"]
          if icon == nil
            icon = Element.new 'icon'
            icon.add_attribute('name', "#{@icon.iconname}")
            context.add_element icon
          end
          icon.elements.each do |symlink|
            symlinks_old += [symlink.text]
          end
          
          symlinks -= symlinks_old
          symlinks.each do |symlink|
            link = Element.new "link"
            link.add_text symlink
            icon.add_element link
          end
        end
        
        f = File.open("#{@dir}/legacy-icon-mapping.xml", "w")
        xml.write(f, -1, false)
        f.close
        
        if !File.exist? "#{@dir}/legacy-icon-mapping.dtd"
          FileUtils.cp("#{ARTIST_SHAREDIR}/buildsys/legacy-icon-mapping.dtd", "#{@dir}/legacy-icon-mapping.dtd")
        end
      end
    end
    
    def parse_icons_data(lines, ext, f)
      icons = add_icon_file_to_buildsys(lines, ext)
      f.puts "icons_DATA = \\"
      icons.each do |i|
        if i == icons.last
          f.puts "#{i}"
        else
          f.puts "#{i} \\"
        end
      end
    end
    
    def parse_symlinks(lines, ext, f)
      symlinks_add = add_symlinks_to_buildsys(lines, ext)
      f.puts "install-data-local: install-iconsDATA"
      symlinks_add.each do |symlink|
        f.puts "#{symlink}"
      end
    end
    
    def add_icon_to_buildsys(icon)
      icon['sizes'].each do |size|
        makefile = "#{@dir}/#{size}/#{icon['context']}/Makefile.am"
        
        if size == 'scalable'
          ext = '.svg'
        else
          ext = '.png'
        end
        
        if File.exist?(makefile)
          lines = IO.readlines("#{makefile}")
        else
          lines = IO.readlines("#{ARTIST_SHAREDIR}/buildsys/Makefile.context.in")
        end
        f = File.new(makefile, "w")
        
        icons_data = false
        extra_dist = false
        install_data_local = false
        noprint = false

        lines.each do |line|
          line.gsub!("@ext@", ext)
          line.gsub!("@size@", size)
          line.gsub!("@context@", icon['context'])
          if line.rstrip == ""
            noprint = false
          elsif line.include?("icons_DATA =")
            noprint = true
            icons_data = true
            parse_icons_data(lines, ext, f)
          elsif line.include?("EXTRA_DIST")
            extra_dist = true
          elsif line.include?("install-data-local")
            noprint = true
            install_data_local = true
            parse_symlinks(lines, ext, f)
          end
          if !noprint
            f.puts "#{line}"
          end
        end
        
        if !icons_data
          parse_icons_data(lines, ext, f)
          f.puts ""
        end
  
        if !extra_dist
          f.puts "EXTRA_DIST = $(icons_DATA)"
          f.puts ""
        end
  
        if !install_data_local & !@icon.symlinks[:optional].empty?
          parse_symlinks(lines, ext, f)
          f.puts ""
        end
  
        f.close

      end
      
    end
    
    def add_icon_none
      icon = @icon.export(@dir)
      create_symlinks(icon)
      
      if File.exist?("#{@source_dir}/#{@icon.iconname}.svg")
        FileUtils.rm("#{@source_dir}/#{@icon.iconname}.svg")
      end
      FileUtils.mkdir_p("#{@source_dir}/one-canvas") unless File.exist?("#{@source_dir}/one-canvas")
      FileUtils.cp("#{@icon.filename}", "#{@source_dir}/one-canvas/#{@icon.iconname}.svg")
    end
    
    def add_icon_binary
      icon = @icon.export(@dir)
      add_icon_to_buildsys(icon)

      if File.exist?("#{@source_dir}/#{@icon.iconname}.svg")
        FileUtils.rm("#{@source_dir}/#{@icon.iconname}.svg")
      end
      FileUtils.mkdir_p("#{@source_dir}/one-canvas") unless File.exist?("#{@source_dir}/one-canvas")
      FileUtils.cp("#{@icon.filename}", "#{@source_dir}/one-canvas/#{@icon.iconname}.svg")
    end
    
    def add_icon_source
      FileUtils.cp("#{@icon.filename}", "#{@dir}/src/#{@icon.iconname}.svg")
      add_symlinks
    end
    
    public 
    
    # Open or create buildsys in target directory
    #
    # target_dir::
    #  directory in which the build sys is supposed to be located
    #
    # options:
    #
    #  :source_dir => '/path/to/dir/where/to/store/sources'
    #  :create => set if you want to create new buildsys
    #  :theme_name => 'theme name'
    #  :theme_version => 'theme version'
    #  :theme_comment => 'theme comment'
    #  :theme_dir => 'directory name'
    #  :theme_bugzilla => 'theme bugzilla'
    #  :theme_inherits => 'theme inherits, defaults to gnome'
    def initialize(target_dir,opts={})
      @dir = target_dir
      if opts[:source_dir]
        @source_dir = opts[:source_dir]
      else
        @source_dir = target_dir
      end
      @theme={:name => 'Foo Icon Theme', :version => '0.0.1', :comment => 'Example Icon Theme', :dir => 'foo-icons', :bugzilla => '', :inherits => 'gnome'}

      @theme[:name] = opts[:theme_name] if opts[:theme_name]
      @theme[:version] = opts[:theme_version] if opts[:theme_version]
      @theme[:comment] = opts[:theme_comment] if opts[:theme_comment]
      @theme[:dir] = opts[:theme_dir] if opts[:theme_dir]
      @theme[:bugzilla] = opts[:theme_bugzilla] if opts[:theme_bugzilla]
      @theme[:inherits] = opts[:theme_inherits] if opts[:theme_inherits]
      
      if opts[:create]
        @buildtype = :source
        initialize_build_sys
      else
        if File.exist?("#{@dir}/index.theme")
          if File.exist?("#{@dir}/Makefile.am")
            @buildtype = :binary
          else
            @buildtype = :none
          end
        else
          if File.exist?("#{@dir}/src")
            @buildtype = :source
          else
            @buildtype = :none
          end
        end
      end
    end
    
    # Add new icon and symlinks to it
    #
    # filename::
    #  file in which the icon is stored
    #
    # symlinks::
    #  non-standard symlinks to be added for the icon
    def add_icon(filename,symlinks=[])
      @icon = Icon.new(filename)
      @icon.add_symlinks(symlinks)
      case @buildtype
        when :none
          add_icon_none
        when :binary
          add_icon_binary
        when :source
          add_icon_source
      end
      @icon.iconname
    end
  end
end
