# gitrepo.rb: Supporting library for icon artists, simplifying working with git
#
# Copyright (C) 2008, 2009 Martin Sourada
# 
# This script is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This script is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this script; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

require 'rubygems'
require 'git'

module IconArtist
  class GitRepo
    # Clone existing repository into new one
    #
    # repository::
    #  path to existing repository
    #
    # name::
    #  name/path to the new repository
    def self.clone(repository,name)
      working_dir = File.expand_path(name)
      if system("git clone #{repository} #{working_dir}")
        options = {:working_dir => working_dir}
        git = self.new(options)
        git.update_branches
        git
      end
    end

    # Checkout branch
    def checkout(branch = 'master')
      system("cd #{@working_dir} && git checkout #{branch}")
    end

    # Sync local branches with remote branches
    def update_branches
      branches_to_create = @branches[:remote] - @branches[:local] - ['HEAD']

      branches_to_create.each do |b|
        if system "cd #{@working_dir} && git branch #{b} origin/#{b}"
          @branches[:local] += [b]
        end
      end
    end

    # Do some stuff within a branch then return back to the original branch
    def in_branch
      old_current = @git.current_branch
      yield
      unless @git.current_branch == old_current
        checkout(old_current)
      end
    end
    
    # Branches hash, contains :remote and :local arrays
    def branches
      @branches
    end

    # Add files from the working directory to the git repository
    def add(path='.')
      @git.add(path)
    end

    # Commit changes
    #
    # message::
    #  commit message
    def commit(message)
      @git.commit(message)
    end

    # Make local repository up2date
    def update
      update_branches
      in_branch do
        @branches[:local].each do |b|
          checkout(b)
          system("cd #{@working_dir} && git pull #{@git.remote.name}")
        end
      end
    end

    # Works like git config
    def config(name = nil, value = nil)
      @git.config(name,value)
    end

    # Open git repository
    #
    # options:
    #  :working_dir => '/path/to/git/repository'
    def initialize(opts={})
      if opts[:working_dir]
        @working_dir = opts[:working_dir]
      else
        @working_dir = File.expand_path('.')
      end

      @git = Git.open(@working_dir)

      @branches = {:remote => [], :local => []}

      @git.branches.remote.each do |b|
        parts = b.full.split(' -> ')[0].split('/')
        parts -= ["remotes"]
        if parts[1]
          @branches[:remote] += [parts[1]]
        else
          @branches[:remote] += [parts[0]]
        end
      end

      @git.branches.local.each do |b|
        @branches[:local] += [b.name]
      end
    end
    
    # Push changes to remote repository
    #
    # remote::
    #   remote repository
    #
    # branch::
    #   branch to push
    def push(remote='origin', branch='master')
      if @branches[:local].include? branch
        @git.push(remote, branch)
      else
        puts "Warning: branch #{branch} does not seem to exist, nothing pushed."
      end
    end

  end
end
