# icon.rb: Supporting library for icon artists, icon generation and
# check utilities
#
# Copyright (C) 2008, 2009 Martin Sourada
# 
# This script is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This script is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this script; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

require "icon-artist/theme-settings"
require "fileutils"
require "rexml/document"

include REXML
include FileUtils

module IconArtist
  INKSCAPE = '/usr/bin/inkscape'
  
  class Icon
  
    private

    def Icon.get_templatedir
      config = File.expand_path("~/.config/user-dirs.dirs")
      dir = "~/Templates"
      if File.exist?(config)  
        f = File.open(config,"r")
          dirs = IO.readlines("#{config}")
          dirs.each do |line|
            if line.include?("XDG_TEMPLATES_DIR=")
              dir = line.gsub("XDG_TEMPLATES_DIR=","").delete('"').rstrip.gsub("$HOME","~")
            end
          end
        f.close
      end
      return dir
    end
    
    def Icon.check_context(context)
      if !THEME_CONTEXTS.include? context
        puts "Error: Unknown icon context '#{context}'. \nContext must be one of: "
        p THEME_CONTEXTS
        return FALSE
      end

      return TRUE
    end

    def Icon.check_document_metadata(svg, filename)
      # Check for icon name
      element = svg.root.elements['//cc:Work/dc:title']
      if element
        title = element.text
        if !title
          puts "Error: Missing document metadata title (icon name)."
          return FALSE
        else
          iconname = File.basename(filename, ".svg")
          if title != iconname
            puts "Warning: Icon name in document metadata differs from file name."
            puts "Icon name in document metadata is: #{title}"
            puts "Icon file name (without extension) is: #{iconname}"
            @@warnings += 1
          end  
        end
      else
        puts "Error: Missing document metadata title (icon name)."
        return FALSE
      end

      # Check for icon context
      element = svg.root.elements['//dc:description']
      if element
        context = element.text
        if !context
          puts "Error: Missing document metadata description (icon context)."
          return FALSE
        else
          if !check_context(context)
            return FALSE
          end
        end
      else
        puts "Error: Missing document metadata description (icon context)."
        return FALSE
      end
    
      return TRUE
     
    end

    def Icon.check_layer_plate(svg)
      # Check if plate layer is present and hidden
      plate_present = FALSE
      svg.root.each_element("//g[@inkscape:label='plate']") do |plate|
        plate_present = TRUE
        style = plate.attributes['style']
        if style != "display:none"
          puts "Error: Plate layer is not hidden."
          return FALSE
        end
      end

      if !plate_present
        puts "Error: Plate layer is missing."
        return FALSE
      end

      # Check for extra sizes/duplicates in plate layer
      plate_sizes = ['scalable']
      svg.root.each_element("//g[@inkscape:label='plate']/rect") do |icon|
        size = "#{icon.attributes['width']}x#{icon.attributes['height']}"
        if THEME_SIZES.include? "#{size}"
          if plate_sizes.include? "#{size}"
            puts "Error: Size #{size} detected multiple times."
            return FALSE
          else
            plate_sizes += ["#{size}"]
          end
        else
          puts "Error: Unknown icon size present in plate layer: #{size}."
          return FALSE
        end
      end
    
      # Check for missing sizes in plate layer
      THEME_SIZES.each do |icon|
        if !plate_sizes.include? icon
          puts "Warning: Size #{icon} missing from plate layer."
          @@warnings += 1
        end
      end

      # Check for scalable icon canvas
      scalable = svg.root.elements["//g[@inkscape:label='plate']/rect[@inkscape:label='scalable']"]
      if !scalable
        puts "Error: Missing element with inkscape:label='scalable' in plate layer."
        return FALSE
      end

      return TRUE
    end
    
    def Icon.check_layer_artwork(svg)
      # Check if artwork layer is present
      artwork = svg.root.elements["//g[@inkscape:label='artwork']"]
      if !artwork
        puts "Error: Artwork layer is missing."
        return FALSE
      end

      # Check for scalable icon artwork
      scalable = svg.root.elements["//g[@inkscape:label='artwork']/g[@inkscape:label='scalable']"]
      if !scalable
        puts "Error: Missing object group with inkscape:label='scalable' in artwork layer."
        return FALSE
      end

      return TRUE    
    end
    
    def export_pngs(export_dir,sizes)
      export_sizes = []
      
      @svg.root.each_element("//g[@inkscape:label='plate']/rect") do |icon|
        size = "#{icon.attributes['width']}x#{icon.attributes['height']}"
        if THEME_SIZES.include?(size) and sizes.include?(size)
          puts "Exporting #{size} size..."
          icon_dir = "#{export_dir}/#{size}/#{@context}"
          FileUtils.mkdir_p(icon_dir) unless File.exist?(icon_dir)
          if system "#{INKSCAPE} -i #{icon.attributes['id']} -e #{icon_dir}/#{@iconname}.png #{@filename} > /dev/null 2>&1"
            export_sizes += ["#{size}"]
          end
        end
      end
      
      return export_sizes
    end
   
    def export_scalable(export_dir)
      puts "Making scalable size..."
      sizes = []
      
      plate = @svg.root.elements["//g[@inkscape:label='plate']/rect[@inkscape:label='scalable']"]
      x = plate.attributes['x']
      y = plate.attributes['y']

      artwork = @svg.root.elements["//g[@inkscape:label='artwork']/g[@inkscape:label='scalable']"]
      #XXX Need update to work with icons that already has transform
      translate = "translate(-#{x},-#{y})"
      artwork.add_attribute(Attribute.new('transform', translate))

      canvas = @svg.root.elements["/svg"]
      canvas.add_attribute(Attribute.new('width', '48'))
      canvas.add_attribute(Attribute.new('height', '48'))

      plate = @svg.root.elements["//g[@inkscape:label='plate']"]
      plate.elements.delete_all('')

      @svg.root.each_element("//g[@inkscape:label='artwork']/*") do |obj|
        label = obj.attributes['inkscape:label']
        if (label != 'scalable')
          obj.parent.delete_element(obj)
        end
      end

      icon_dir = "#{export_dir}/scalable/#{@context}"
      out ="#{icon_dir}/#{@iconname}.svg"
      FileUtils.mkdir_p(icon_dir) unless File.exist?(icon_dir)

      f = File.open(out,"w")
      @svg.write(f, -1, false)
      f.close

      if system "#{INKSCAPE} --vacuum-defs -l #{out} -f #{out} > /dev/null 2>&1"
        sizes += ['scalable']
      end
      
      return sizes
    end
    
    def initialize_symlinks
      icon_mapping = "/usr/share/icon-naming-utils/legacy-icon-mapping.xml"
      @symlinks_def = []
      @symlinks_opt = []

      icon_map = Document.new(File.new(icon_mapping, 'r'))

      icon = icon_map.root.elements["//context[@dir='#{@context}']/icon[@name='#{@iconname}']"]

      if icon != nil
        icon.each_element('link') do |link|
          @symlinks_def = @symlinks_def + [link.text]
        end
      end
      @symlinks_initialized = true
    end

    public
    
    # Open new icon from set file
    #
    # filename::
    #  path to a file that contains the icon
    #
    # options::
    #    :skip_check => don't check if the icon adheres one-canvas guidelines
    def initialize(filename,opts={})
      @symlinks_initialized = false
      @filename = filename
      @svg = Document.new(File.new("#{filename}", 'r'))

      @@warnings = 0
      unless opts[:skip_check]
        if !Icon.check_document_metadata(@svg, @filename)
          raise "Failed. Check above messages for more info."
        end
  
        if !Icon.check_layer_plate(@svg)
          raise "Failed. Check above messages for more info."
        end
  
        if !Icon.check_layer_artwork(@svg)
          raise "Failed. Check above messages for more info."
        end
      end
      
      @iconname = @svg.root.elements['//cc:Work/dc:title'].text
      @context = @svg.root.elements['//dc:description'].text

      puts "Icon #{@filename} loaded successfully."
      if @@warnings > 0
        if @@warnings == 1
          puts "There was #{@@warnings} warning during icon load."          
        else
          puts "There were #{@@warnings} warnings during icon load."
        end
      end
    end

    # Create new icon with set name and context using selected template
    #
    # iconname::
    #  name of the icon, preferably following freedesktop.org 
    #  standards, to be created
    #
    # context::
    #  context of the icon, see THEME_CONTEXTS constant for more info
    #
    # template:: 
    #  select custom template, either template name (without .svg 
    #  extension) that is located in your XDG User Dirs Templates dir or in 
    #  /usr/share/icon-artist/templates/#{template}.svg, or a filename of a 
    #  custom template (with the .svg extension)
    def self.create(iconname, context, template="default")
      userdir_templates = Icon.get_templatedir
      @template = File.expand_path("#{template}")
      if !File.exist?(@template)
        @template = File.expand_path("#{userdir_templates}/#{template}.svg")
        if !File.exist?(@template)
          @template = "#{ARTIST_SHAREDIR}/templates/#{template}.svg"
          if !File.exist?(@template)
            raise "Could not find either '#{File.expand_path(template)}', '#{File.expand_path(userdir_templates)}/#{template}.svg' or '#{ARTIST_SHAREDIR}/templates/#{template}.svg'."
          end
        end
      end
      
      @iconname=iconname
      @context=context
      @filename=File.expand_path("#{@iconname}.svg")
      
      if File.exist?(@filename)
        raise "Selected icon (#{@filename}) already exists. \nPlease set another icon name."
      end
      
      if !(THEME_CONTEXTS.include?(@context))
        out = "Context must be one of: "
        THEME_CONTEXTS.each {|el| out = out + "#{el}, "}
        out = (out + "\n").gsub(", \n", ".\n")
        raise out
      end
      
      @svg = Document.new(File.new("#{@template}", 'r'))
      
      @svg.root.attributes.delete_all("inkscape:export-filename")
      @svg.root.attributes.delete_all("sodipodi:docname")
    
      element = @svg.root.elements["metadata/rdf:RDF/cc:Work"]

      element.elements.delete_all("dc:title")
      element.elements.delete_all("dc:description")

      title = Element.new "dc:title"
      title.add_text "#{@iconname}"
      element.add_element title

      description = Element.new "dc:description"
      description.add_text "#{@context}"
      element.add_element description

      f = File.open(@filename,"w")
      @svg.write(f, -1, false)
      f.close

      puts "New icon #{@filename} created."
      self.new(@filename, :skip_check => true)
    end
    
    # Check if icon in given file follows one-canvas standards
    #
    # filename:: 
    #  path to the file with the icon
    def self.check(filename)
      @@warnings = 0
      @filename = filename
      @svg = Document.new(File.new("#{filename}", 'r'))
      puts "Checking document metadata..."
      if Icon.check_document_metadata(@svg, @filename)
        puts "Done."
      else
        raise "Failed. Check above messages for more info."
      end

      puts "Checking plate layer..."
      if Icon.check_layer_plate(@svg)
        puts "Done."
      else
        raise "Failed. Check above messages for more info."
      end

      puts "Checking artwork layer..."
      if Icon.check_layer_artwork(@svg)
        puts "Done."
      else
        raise "Failed. Check above messages for more info."
      end
      
      puts "\nCheck complete. No fatal errors found."
      if @@warnings > 0
        if @@warnings == 1
          puts "There was #{@@warnings} warning."          
        else
          puts "There were #{@@warnings} warnings."
        end
      end
    end
    
    # Export the PNGs and SVGs to selected directory using standard icon theme
    # directory structure
    #
    # dir::
    #  export directory
    #
    # sizes::
    #  set sizes to export
    def export(dir=".",sizes=THEME_SIZES)
      export_dir = File.expand_path(dir)
      exported_sizes = export_pngs(export_dir,sizes)
      if sizes.include?('scalable')
        exported_sizes += export_scalable(export_dir)
      end
      puts "Icon #{@iconname} exported successfully."
      icon = {
        "name" => @iconname,
        "context" => @context,
        "sizes" => exported_sizes
      }
      return icon
    end
    
    # Add symlinks to the icon
    #
    # symlinks::
    #  array of symlinks to be added
    def add_symlinks(symlinks)
      initialize_symlinks unless @symlinks_initialized
      symlinks.each do |symlink|
        if !@symlinks_def.include?(symlink) & !@symlinks_opt.include?(symlink)
          @symlinks_opt += [symlink]
        end
      end
    end
    
    # Accessor to symlinks. Returs a hash containing :default and :optional
    # symlinks. :default symlinks are found using icon-naming-utils.
    def symlinks
      initialize_symlinks unless @symlinks_initialized
      return {:default => @symlinks_def, :optional => @symlinks_opt}
    end
    
    # Accessor to icon contex
    def context
      @context
    end
    
    # Accessor to icon name
    def iconname
      @iconname
    end
    
    # Accessor to icon filename
    def filename
      @filename
    end
    
  end
end
