# icontheme.rb: Supporting library for icon artists icon theme related
# stuff like adding icons/folders to the theme
#
# Copyright (C) 2008, 2009 Martin Sourada
# 
# This script is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This script is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this script; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

require "icon-artist/theme-settings"
require "fileutils"
require "rexml/document"

include REXML
include FileUtils

module IconArtist
  class IconTheme
  
    private
    
    DEFAULT_DIR_SETS = {
      'actions' => 'Actions', 
      'animations' => 'Animations', 
      'apps' => 'Applications', 
      'categories' => 'Categories', 
      'devices' => 'Devices', 
      'emblems' => 'Devices', 
      'emotes' => 'Emotes', 
      'intl' => 'International', 
      'mimetypes' => 'MimeTypes',
      'places' => 'Places', 
      'status' => 'Status'
    }
    
    def dirs_parse(directories)
      @dirs = directories.scan(/[\w+\/]+/)
    end
    
    def index_add_dir(dir)
      if !@dirs.include?(dir)
        @dirs += [dir]
        @dir_settings[dir] = []

        aux = dir.scan(/\w+/)
        size = aux[0]
        if size == "scalable"
          icon_size = "48"
        else 
          icon_size = size.scan(/\d+/)[0]
        end
        context = DEFAULT_DIR_SETS[aux[1]]
        
        @dir_settings[dir] += ["Size=#{icon_size}\n"]
        @dir_settings[dir] += ["Context=#{context}\n"]
        if size == "scalable"
          @dir_settings[dir] += ["Type=Scalable\n"]
          @dir_settings[dir] += ["MinSize=24\n"]
          @dir_settings[dir] += ["MaxSize=256\n"]
        else
          @dir_settings[dir] += ["Type=Fixed\n"]
        end

      end
    end
    
    def get_dirs(icons)
      dirs = []
      icons.each do |icon|
        icon['sizes'].each do |size|
          dir = "#{size}/#{icon['context']}"
          if !dirs.include?(dir)
            dirs += [dir]
          end
        end
      end
      return dirs
    end
    
    def index_add_dirs(dirs)
      dirs.each do |dir|
        index_add_dir(dir)
      end
    end

    def index_create(dirs)
      @dirs = []
      @dir_settings = {}
      
      dirs.each do |dir|
        index_add_dir(dir)
      end
    end
    
    def index_parse(file)
      index = IO.readlines(file)
      @dirs = []
      @dir_settings = {}
      set = ""
      reading = false
      
      index.each do |line|
        if line.include?("_Name=")
          @name = line.gsub("_Name=","").rstrip
        elsif line.include?("Name=")
          @name = line.gsub("Name=","").rstrip
        elsif line.include?("_Comment=")
          @comment = line.gsub("_Comment=","").rstrip
        elsif line.include?("Comment=")
          @comment = line.gsub("Comment=","").rstrip
        elsif line.include?("Inherits=")
          @inherits = line.gsub("Inherits=","").rstrip
        elsif line.include?("Directories=")
          dirs_parse(line.gsub("Directories=",""))
        elsif line.include?("[")
          set = line.delete("[]").rstrip
          if @dirs.include? set
            reading = true
            @dir_settings[set] = []
          end
        elsif line.rstrip == ""
          reading = false
          set = ""
        elsif reading
          @dir_settings[set] += [line]
        end
      end
    end
    
    def print_dirs
      dirs = @dirs.join(",")
      return dirs
    end
    
    def print_dir_settings
      dir_sets = ""
      @dirs.each do |dir|
        dir_sets += "[#{dir}]\n"
        @dir_settings[dir].each do |dir_set|
          dir_sets += dir_set
        end
        dir_sets += "\n"
      end
      return dir_sets
    end
    
    def index_initialize(icons, create=false)
      dirs = get_dirs(icons)
      if create or !File.exist?("#{@dir}/#{@index_name}")
        index_create(dirs)
      else
        index_parse("#{@dir}/#{@index_name}")
        dirs.each do |dir|
          index_add_dir(dir)
        end
      end
    end
    
    def index_save
      index = IO.readlines("#{ARTIST_SHAREDIR}/theme/#{@index_name}.in")
      f = File.new("#{@dir}/#{@index_name}", "w+")
      index.each do |line|
        line.gsub!("@name@",@name)
        line.gsub!("@comment@",@comment)
        line.gsub!("@inherits@",@inherits)
        line.gsub!("@directories@",print_dirs)
        line.gsub!("@dir_settings@",print_dir_settings)
        f.puts line
      end
      f.close
    end
    
    public
    
    # Open icon theme in selected dir. If not present, or +create+ set to true
    # create a new one
    #
    # dir::
    #  directory in with the icon theme is located
    #
    # options:
    #
    #  :create => if true forces creating of new icon theme
    #  :icons => icons to be added, array of hashes containg 'name', 'context' and 'sizes'
    #  :name => 'theme version'
    #  :comment => 'theme comment'
    #  :inherits => 'theme inherits, defaults to gnome'
    #  :translatable => if true we'll use index.theme.in instead
    def initialize(dir, opts = {})
      @dir = dir

      if opts[:create]
        create = opts[:create]
      else
        create = false
      end
      
      if opts[:icons]
        icons = opts[:icons]
      else
        icons = opts[]
      end
      
      if opts[:name]
        @name = opts[:name]
      else
        @name = 'Foo Icon Theme'
      end

      if opts[:comment]
        @comment = opts[:comment]
      else
        @comment = 'Sample icon theme.'
      end

      if opts[:inherits]
        @inherits = opts[:inherits]
      else
        @inherits = 'gnome'
      end
      
      if opts[:translatable]
        @index_name = 'index.theme.in'
      else
        @index_name = 'index.theme'
      end
      
      index_initialize(icons,create)
    end
    
    # Save the changes made to the icon theme
    def save
      FileUtils.mkdir_p(@dir) unless File.exists?(@dir)
      index_save
    end
    
  end
end
